use std::{collections::HashMap, ffi::CStr};

use log::{debug, info, trace, warn};
use pam::{
    constants::{PamResultCode, PAM_PROMPT_ECHO_OFF},
    conv::Conv,
    items::User,
    module::PamHooks,
    pam_try,
};
use syslog::Facility;

struct PamHoneypot;

impl PamHooks for PamHoneypot {
    fn sm_authenticate(
        pamh: &mut pam::module::PamHandle,
        args: Vec<&CStr>,
        _flags: pam::constants::PamFlag,
    ) -> pam::constants::PamResultCode {
        let _ = syslog::init(
            Facility::LOG_USER,
            log::LevelFilter::Info,
            Some("pam_honeypot"),
        );

        trace!("Running pam_honeypot");
        let args: Vec<_> = args.iter().map(|s| s.to_string_lossy()).collect();
        debug!("args = {args:?}");
        let args: HashMap<&str, &str> = args
            .iter()
            .map(|s| {
                let mut parts = s.splitn(2, '=');
                (
                    parts.next().expect("Failed to parse arguments"),
                    parts.next().unwrap_or(""),
                )
            })
            .collect();

        let honey_user = match args.get("user") {
            Some(user) => user,
            _ => return PamResultCode::PAM_IGNORE,
        };
        debug!("honeypot user = {honey_user}");
        let honey_password = match args.get("password") {
            Some(password) => password,
            _ => return PamResultCode::PAM_IGNORE,
        };

        trace!("Getting user from PAM");
        let user: User = pam_try!(pamh.get_item()).unwrap_or(User(c""));
        info!("Checking if entered username is honeypot user");
        if user.to_string_lossy() != *honey_user {
            return PamResultCode::PAM_IGNORE;
        }

        let conv = match pamh.get_item::<Conv>() {
            Ok(Some(conv)) => conv,
            Ok(_) => {
                unreachable!("No conv available");
            }
            Err(err) => {
                return err;
            }
        };
        trace!("Getting password from PAM");
        let password = pam_try!(conv.send(PAM_PROMPT_ECHO_OFF, "Password: "));
        let password = match password {
            Some(password) => Some(pam_try!(password.to_str(), PamResultCode::PAM_AUTH_ERR)),
            _ => None,
        };

        if let Some(password) = password {
            trace!("Checking if password is honeypot password");
            if *honey_password == password {
                warn!("Someone tried to log in with the honeypot credentials");
            } else {
                info!("Entered username is not the honeypot user")
            }
            return PamResultCode::PAM_AUTH_ERR;
        }

        PamResultCode::PAM_IGNORE
    }
}

pam::pam_hooks!(PamHoneypot);
